import { ComponentClass, Component, InputHTMLAttributes } from "react";
/**
 * @see {@link https://github.com/mobxjs/mobx/blob/main/packages/mobx-react/README.md#note-on-using-props-and-state-in-derivations}
 */
export function observePropsState<T extends ComponentClass>(ComponentBaseClass: T): T;
export interface FormComponentProps extends Omit<InputHTMLAttributes<HTMLInputElement>, "onChange"> {
    onChange?: (value: InputHTMLAttributes<HTMLInputElement>["value"], ...extra: any[]) => any;
}
/**
 * @example
 * ```tsx
 * export interface MyFieldProps
 *     extends Omit<InputHTMLAttributes<HTMLInputElement>, 'onChange'> {
 *     onChange?: (value: string) => any;
 * }
 *
 * @observer
 * @observePropsState
 * export class MyField extend FormComponent<MyFieldProps> {
 *     handleChange = ({ currentTarget: { value }}: MouseEvent<HTMLInputElement>) => {
 *         this.innerValue = value;
 *
 *         this.props.onChange?.(this.innerValue);
 *     };
 *
 *     render() {
 *         const { onChange, ...props } = this.props,
 *             { value, handleChange } = this;
 *
 *         return <>
 *             <input {...props} onChange={handleChange} />
 *
 *             <output>{value}</output>
 *         <>;
 *     }
 * }
 * ```
 */
export abstract class FormComponent<P extends FormComponentProps = FormComponentProps, S = {}, SS = any> extends Component<P, S, SS> {
    constructor(props: P);
    ref: import("react").RefObject<HTMLInputElement & HTMLTextAreaElement & HTMLSelectElement>;
    innerValue: P["defaultValue"];
    observedProps: P;
    get value(): string | number | readonly string[];
    reset: () => void;
    componentDidMount(): void;
    componentWillUnmount(): void;
}

//# sourceMappingURL=index.d.ts.map
